# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

import requests
from pydantic import ConfigDict
from requests import Response
from pydantic.dataclasses import dataclass


@dataclass
class Session:
    base_url: str
    session: requests.Session

    __pydantic_config__ = ConfigDict(arbitrary_types_allowed=True)

    def request(self, path: str, **kwargs) -> Response:
        kwargs["url"] = f"{self.base_url}{path}"
        resp = self.session.request(**kwargs)
        if resp.status_code < 400:
            return resp
        elif resp.status_code < 500:
            # A 4XX error happened
            raise NotImplementedError("TODO: Add error reporting")
        elif resp.status_code < 600:
            # A 5XX error happened
            raise NotImplementedError("TODO: Add error reporting")
        return resp

    def get(self, path: str, **kwargs) -> Response:
        return self.request(method="GET", path=path, **kwargs)

    def post(self, path: str, **kwargs) -> Response:
        return self.request(method="POST", path=path, **kwargs)

    def __del__(self) -> None:
        self.session.close()



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

import pytest


@pytest.fixture(
    params=[
        dict(
            user_email="harsh@ikigailabs.io",
            api_key="2gnVFartBD9i2XDt7AhAsAo8WY7",
            base_url="https://dev-api.ikigailabs.io",
        )
    ],
    ids=["dev-api"],
)
def cred(request) -> dict[str, str]:
    return request.param



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

import requests
from pydantic import EmailStr, AnyUrl, Field
from pydantic.dataclasses import dataclass
from ikigai import components
from ikigai.client.session import Session


@dataclass
class Ikigai:
    user_email: EmailStr
    api_key: str
    base_url: AnyUrl = Field(default=AnyUrl("https://api.ikigailabs.io"))
    __session: Session = Field(init=False)

    def __post_init__(self) -> None:
        session = requests.Session()
        session.headers.update({"user": self.user_email, "api-key": self.api_key})
        self.__session = Session(base_url=str(self.base_url), session=session)

    def projects(self) -> list[components.Project]:
        resp = self.__session.get("/component/get-projects-for-user").json()
        projects = [
            components.Project.from_dict(data=project_dict, session=self.__session)
            for project_dict in resp["projects"]
        ]

        return projects



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from typing import Any
from ikigai import Ikigai


def test_client_init(cred: dict[str, Any]) -> None:
    ikigai = Ikigai(**cred)
    assert ikigai


def test_client_projects(cred: dict[str, Any]) -> None:
    ikigai = Ikigai(**cred)
    projects = ikigai.projects()
    assert len(projects) > 0



# SPDX-FileCopyrightText: 2024-present Harsh Parekh <harsh@ikigailabs.io>
#
# SPDX-License-Identifier: MIT

from __future__ import annotations
import sys
from datetime import datetime
from pydantic import BaseModel, EmailStr
from ikigai.client.session import Session

# Multiple python version compatible import for Self
if sys.version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


class Project(BaseModel):
    project_id: str
    name: str
    owner: EmailStr
    description: str
    created_at: datetime
    modified_at: datetime
    last_used_at: datetime
    __session: Session

    @classmethod
    def from_dict(cls, data: dict, session: Session) -> Self:
        self = cls.model_validate(data)
        self.__session = session
        return self
